      subroutine 
     1find(r1,r2,z0,zr,check0,zmiss,ystart1,delta,eiga,isuccess)
      implicit double precision (a-h,o-z)
c
c This subroutine finds the eigenray arrival between angles start1-delta
c and start1 that arrive at the receiver depth.  Reports back sin(eigenangle) in 
c variable eiga, and isuccess = 1 if successful in finding the arrival.  0 if not.
c
c The eigenrays are found by three steps:
c     Plan A: bisection until arrival is within specified criteria (zmiss)
c             After MAX_BISEC bisections and no success it goes to Plan B.
c     Plan B: brute force calculation of N_BRUTE rays if Plan A fails.  
c             After N_BRUTE rays and still no success it goes to Plan C.
c     Plan C: during the calculations of Plan A and Plan B, the closest reception 
c             to the receiver is saved, and this reception is reported back if Plan A
c             and Plan B fail.  This usually means that a gap is in the timefront, so 
c             finding the ray within zmiss is impossible.  In this case,
c             the closest arrival may miss the receiver by O(100+ m) and the
c             travel time may be in error by 10-20 ms (?).
c
c  N.B. isuccess is nearly always 1 with this design.  You may want to change that.
c  isuccess is 0 if something about the ray situation has gone seriously
c  awry.
c
c r1,r2          source, receiver ranges
c z0,zr          source, receiver depth
c check0         sign of latest arrival
c zmiss          the tolerance to missing the receiver depth
c ystart1, delta  looking for arrival between start1-delta and start1
c eiga            the eigenray angle to return
c isuccess       is 1 if found the eigenray, 0 if not
c
c The angles here are actually sin(angle); cos(angle) is calculated from this
c
c  Brian Dushaw
c  Applied Physics Laboratory 
c  University of Washington
c  Seattle, WA 98105
c  (206) 543-1300
c  dushaw@apl.washington.edu
c
c  EIGENRAY Version 3.4
c  September-December 1998
c  Last revised March 2013

       include 'nrays.h'
       double precision check_1,check0,check1
c Need three checks for the three angles of the bisection - these tell
c which side of the receiver depth the arrivals are.

       double precision ystart(4)
       double precision xp(KMAXX),yp(2,KMAXX),deltaX
       integer kount,iprof,kmax

       parameter(MAX_BISEC=10,N_BRUTE=29)
c N_BRUTE seems best to be a prime number so that the brute force
c ray angles will not match any of the bisection angles.
c Note that the bruteforce method is actually looking at MAX_BISEC+N_BRUTE rays.
       include 'constants.h'

          isuccess = 0
        if(abs(delta).lt.1.0d-10) then
         print *,'delta is zero'
         return
        endif

          xmin_angle=1.0d0
          xmin_zmiss=1000.d0
 
          check_1=-check0
          angle1=ystart1-delta
          angle2=ystart1
          angle3=(angle1+angle2)*half
c looking to bisect the angle between the present ray arrival and the
c previous one.

          igetout=0

1111       continue

c  Add a safeguard in case we get trapped in here.
          igetout=igetout+1
          if(igetout.gt.MAX_BISEC)goto 1115

c angle3 has the sin of the angle, actually.  Instead of calculating 
c cos(asin(eigenangle)) get cos(theta) by simple relation to sin(theta).
         temper=angle3*angle3
         ystart(1)=dsqrt(one - temper)
         ystart(2)=angle3
         ystart(3)=z0
         ystart(4)=zero

         call odeint(ystart,r1,r2,xp,yp,deltaX,kount,iprof,kmax)
       if(ystart(3).lt.-half) goto 1115
c This happens when we get a ray trapped at the surface or excessive
c bottom bounces; ystart has no meaning, but is just flagged this way.  
c Kludge for now,  and punt with the brute force method.

            dmiss=dabs(zr-ystart(3))

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
            if(dmiss.le.zmiss)then
c        print *,'found ray in ', igetout, ' bisections'
              eiga=angle3
              isuccess=1
              return
            else
c we missed so now try another bisection....

       if(dmiss.lt.xmin_zmiss) then
           xmin_angle=angle3
           xmin_zmiss=dmiss
       endif

       check1=dsign(one,zr-ystart(3))

       if (dabs(check1-check0).gt.half) then
c have crossed again, and average the latest two angles
           angle1=angle2
           angle2=angle3
           angle3=(angle1+angle2)*half
        elseif (dabs(check1-check0).lt.half
     *.and.dabs(check_1-check0).lt.half) then
c Because of nonlinearity, all angles are now on the same side of the
c receiver depth...back up and try again...
c We start with ystart1 and z0 as we did at the very beginning, but
c we are bisecting against the most latest predictions so we are still
c trying new angles.
c
         temper=ystart1*ystart1
         ystart(1)=dsqrt(one - temper)
         ystart(2)=ystart1
         ystart(3)=z0
         ystart(4)=zero

         call odeint(ystart,r1,r2,xp,yp,deltaX,kount,iprof,kmax)
       if (ystart(3).lt.-half) goto 1115
c This happens when we get a ray trapped at the surface or excessive
c bottom bounces.  Kludge for now, and just go to the brute force method.

         check1=dsign(one,zr-ystart(3))
         if (dabs(check1-check0).lt.half) then
c the angle ystart1-delta is the one on the other side...
c and angle3, ystart1-delta span the receiver
           angle1=angle3
           angle2=ystart1-delta
           angle3=(angle1+angle2)*half
           check1=-check1
         else
           angle1=angle3
           angle2=ystart1
           angle3=(angle1+angle2)*half
         endif
       else
c Have not crossed, so use the older angle with the most recent one
           if(ystart(3).gt.-half)angle2=angle3
           angle3=(angle1+angle2)*half
           check0=-check0
       endif
           check_1=check0
           check0=check1
           goto 1111
          endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c We have now given up on bisection finding the eigenray and going to brute
c force.  However, if xmin_zmiss is still 1000 m, we will not do this.
1115        continue
c       print *,'Ray not found, using brute force, then closest approach'

c If we haven't gotten any closer than 1000 m, then there is no point in proceding.
c isuccess is zero by default
            if(abs(xmin_zmiss).gt.999.0) return

              dd1=delta/N_BRUTE
              ystart2=ystart1-delta
c Do not want to redo the angle at ystart1-delta (which is guaranteed 
c not to be within tolerance since we are in this subroutine)
              do i=1,(N_BRUTE-1)
            ystart2=ystart2+dd1

           temper=ystart2*ystart2
           ystart(1)=dsqrt(one - temper)
           ystart(2)=ystart2
           ystart(3)=z0
           ystart(4)=zero

           call odeint(ystart,r1,r2,xp,yp,deltaX,kount,iprof,kmax)
           if (ystart(3).gt.-half) then
c Skip all rays for which ystart(3)=-1, which indicates a failed ray.
             dmiss=dabs(zr-ystart(3))

             if(dmiss.le.zmiss)then
               eiga=ystart2
               isuccess=1
               return
             endif

             if(dmiss.lt.xmin_zmiss) then
                xmin_angle=ystart2
                xmin_zmiss=dmiss
             endif
          endif
              enddo

      print *,
     *'Closest ray missed receiver by ',int(xmin_zmiss+half),' meters'
            eiga=xmin_angle
            isuccess=1
            return

         end
